
%---------------------------------------------------------------------
%---  This code calibrates the measured backscatter response of a point 
%---  target using the STCT calibration technique. The elements of the 
%---  scattering matrix of the target must be measured coherently. 
%---
%--- The code loads all necessary measured data from files and manipulates
%--- them before performing calibration steps.
%---
%--- It requires two measured data files for the cal_target (sphere). One
%--- of the files contains the measured sphere data and the 2nd file 
%--- contains the measured background data (without sphere).
%---
%--- It also requires two data files for the unknown target (or target
%--- under test) intended for calibration. One file is for measured
%--- target data and the 2nd file holds background data.
%---
%--- All files hold the measured complex scattering matrix. Background data may
%--- be ignored, however the user must modify the code to permit this step.
%---
%--- Additional information that must be inputted by the user are sphere size, 
%--- range to sphere, range to unknown target, start and stop frequencies, 
%--- and number of frequency points used.
%---
%---  Written for matlab use by:  Adib Nashashibi              6/27/2017
%--- 
%---------------------------------------------------------------------

 clear;
 
 r2d = 180.0/pi ;
 nf_cal = 401; % number of frequency points used in DAQ/gating steps
 fstart = 8.75; % in GHz
 fstop = 10.25; % in GHz

 diam = 8; % diameter of calibration sphere in inches 

%--- enter file names for the sphere and its background
 m_sph_file = 'cal_target_data.dat'; % file name storing measured sphere data (reference target)
 m_s_bkg_file = 'cal_target_bkg.dat'; % file name storing measured background data 
 
 r_ref = 15; % one-way distance from radar to sphere in meters 
 
 
 %--- enter the files names for the target with unknown RCS and its background.
 
 m_trgt_file = 'target_12inchSphere_data.dat'; % target file name
 m_t_bkg_file = 'target_12inchSphere_bkg.dat'; %its background file name
 r_target = 15; % one-way distance from radar to unknown target in meters 
 
 nang = 1; %number of samples (or angles) measured for the target (minimum:1)
 
 
 %-- Dimension and initialize useful arrays needed for computation
 m_sph_fld_vv = zeros(nf_cal,1); m_sph_fld_hh= m_sph_fld_vv; %initialize variables
 m_sph_fld_vh= m_sph_fld_vv; m_sph_fld_hv= m_sph_fld_vv; 
 
 m_bkg_fld_vv = zeros(nf_cal,1); 
 m_bkg_fld_vh = m_bkg_fld_vv; m_bkg_fld_hv = m_bkg_fld_vv;
 m_bkg_fld_hh = m_bkg_fld_vv;
 
 m_trgt_fld_vv= zeros(nf_cal, nang); m_trgt_fld_hh=m_trgt_fld_vv;
 m_trgt_fld_hv=m_trgt_fld_vv;m_trgt_fld_vh=m_trgt_fld_vv;
 
 mtmp = zeros(4, nf_cal*nang);
 
 scat_cal=zeros(nf_cal, 2,2,nang); 
 
 %%%%==============================================%%%
 %--- load the calib. data files.
 %
 m_sphere_data = load(m_sph_file,'-ascii'); %load the sphere data
 m_bkg_data = load(m_s_bkg_file,'-ascii'); %load the background data

 m_sph_fld_vv(:) = m_sphere_data(:,3) + 1i* m_sphere_data(:,4); %sort data vv-pol 
 m_sph_fld_vh(:) = m_sphere_data(:,5) + 1i* m_sphere_data(:,6); %sort data vh-pol 
 m_sph_fld_hv(:) = m_sphere_data(:,7) + 1i* m_sphere_data(:,8); %sort data hv-pol 
 m_sph_fld_hh(:) = m_sphere_data(:,9) + 1i* m_sphere_data(:,10); %sort data hh-pol 

 m_bkg_fld_vv(:) = m_bkg_data(:,3) + 1i* m_bkg_data(:,4);
 m_bkg_fld_vh(:) = m_bkg_data(:,5) + 1i* m_bkg_data(:,6);
 m_bkg_fld_hv(:) = m_bkg_data(:,7) + 1i* m_bkg_data(:,8);
 m_bkg_fld_hh(:) = m_bkg_data(:,9) + 1i* m_bkg_data(:,10);
 
 m_ref_fld_vv = m_sph_fld_vv - m_bkg_fld_vv; %coherent subtraction of background from sphere data
 m_ref_fld_vh = m_sph_fld_vh - m_bkg_fld_vh;
 m_ref_fld_hv = m_sph_fld_hv - m_bkg_fld_hv;
 m_ref_fld_hh = m_sph_fld_hh - m_bkg_fld_hh;
 
 
 
 %%%%==============================================%%%
 %--- load the target (unknown RCS) data files.

 m_trgt_data = load(m_trgt_file,'-ascii'); %load the target data
 m_t_bkg_data = load(m_t_bkg_file,'-ascii'); %load the background data

 mtmp(1,:) = (m_trgt_data(:,3) +1i*m_trgt_data(:,4)) ...
             - (m_t_bkg_data(:,3) + 1i* m_t_bkg_data(:,4));
 mtmp(2,:) = (m_trgt_data(:,5) +1i*m_trgt_data(:,6)) ...
             - (m_t_bkg_data(:,5) + 1i* m_t_bkg_data(:,6));
 mtmp(3,:) = (m_trgt_data(:,7) +1i*m_trgt_data(:,8)) ...
             - (m_t_bkg_data(:,7) + 1i* m_t_bkg_data(:,8));
 mtmp(4,:) = (m_trgt_data(:,9) +1i*m_trgt_data(:,10)) ...
             - (m_t_bkg_data(:,9) + 1i* m_t_bkg_data(:,10));
         
 %-- cast into separate samples
  for ii = 1: nang
      n1 = (ii-1)*nf_cal + 1;
      n2 = (n1-1) + nf_cal;
      
      m_trgt_fld_vv(:,ii) = mtmp(1,n1:n2); 
      m_trgt_fld_vh(:,ii) = mtmp(2,n1:n2); 
      m_trgt_fld_hv(:,ii) = mtmp(3,n1:n2); 
      m_trgt_fld_hh(:,ii) = mtmp(4,n1:n2); 
  end
  
  
%---- calculate theoretical sphere's response
%-- The sphere is PEC and the sought response is in far field and
%-- backscatter. 

s_ref = RCSmetallic_sphere(diam, nf_cal, fstart, fstop); %call sphere code

delta = (fstop - fstart)/(nf_cal-1); % delta_freq steps in VNA
freq = 1:1: nf_cal;
freq = (fstart-delta) + freq*delta; %freq vector
k0 = 2*pi * freq' / 0.2998; % wavenumber

figure(1)
plot(freq, 10*log10(s_ref.^2 * 4*pi) )
grid
xlabel('freq in GHz')
ylabel('\sigma (dBsm)')
title('Theoretical RCS of metallic sphere')
axis([ fstart fstop  -40 0])
  
%------------------------------------------------------------------------
%c--------------Actual Calibration begins here ---------

%-- calculate the error coefficients using sphere data
a = m_ref_fld_vh .* m_ref_fld_hv ./(m_ref_fld_vv .* m_ref_fld_hh); 

c = (1 - sqrt(1 - a)) ./ sqrt(a);

const1 = c.^2;
const2 = 1 ./(1-const1).^2 .* s_ref .*(r_target/r_ref).^2;
const2 = const2 .* exp(-2 .*1i .*k0 .*(r_ref - r_target));
const3 = (1 + const1).^2;

rt11 = m_ref_fld_vv ./ s_ref ./ (1 + c.^2) ; 
rt12 = m_ref_fld_vh ./ s_ref ./ 2.0 ./ c;
rt21 = m_ref_fld_hv ./ s_ref ./ 2.0 ./ c;
rt22 = m_ref_fld_hh ./ s_ref ./ (1 + c.^2);

figure(2)
subplot(2,1,1)
plot(freq, abs(rt11), freq, abs(rt12), freq, abs(rt21),freq, abs(rt22))
    set(gca, 'FontSize', 14, 'FontWeight', 'b')
%title('cylinder','FontSize',14, 'FontWeight','b')
    xlabel('Frequency(GHz)','FontSize',14, 'FontWeight','b')
    ylabel('RT_{ij}', 'FontSize',14, 'FontWeight','b')
    ch = get(gca, 'Children');
    set(ch(4),'LineWidth', 2.0, 'LineStyle','-', 'Color','k')
    set(ch(3),'LineWidth', 2.0, 'LineStyle','-', 'Color','b')
    set(ch(2),'LineWidth', 2.0, 'LineStyle','-','Color','g') 
    set(ch(1),'LineWidth', 2.0, 'LineStyle','-','Color','r') 
    legend('RT_{11}', 'RT_{12}', 'RT_{21}', 'RT_{22}', 3)
    grid

subplot(2,1,2)
    plot(freq, abs(c).^2)
    set(gca, 'FontSize', 14, 'FontWeight', 'b')
%title('cylinder','FontSize',14, 'FontWeight','b')
    xlabel('Frequency(GHz)','FontSize',14, 'FontWeight','b')
    ylabel('|c|^2', 'FontSize',14, 'FontWeight','b')
    ch = get(gca, 'Children');
    set(ch(1),'LineWidth', 2.0, 'LineStyle','-','Color','r') 
    grid

%-- calibrate the measured target data (with unknown rcs)

for ii = 1: nang
    r11 = m_trgt_fld_vv(:,ii) ./ m_ref_fld_vv(:);
    r12 = m_trgt_fld_vh(:,ii) ./ m_ref_fld_vh(:);
    r21 = m_trgt_fld_hv(:,ii) ./ m_ref_fld_hv(:);
    r22 = m_trgt_fld_hh(:,ii) ./ m_ref_fld_hh(:);

    scat_cal(:,1,1,ii) = const2 .* (-2.0 .* const1 .* (r12 + r21) ...
        + const3 .* (r11 + const1 .* r22) ); %calibrated vv-pol
    scat_cal(:,2,2,ii) = const2 .* (-2.0 .* const1 .* (r12 + r21) ...
                  + const3 .* (r22 + const1 .* r11) );% hh-pol
    scat_cal(:,1,2,ii) = const2 .* c .* (2.0 .* (const1 .* r21 + r12) ...
                  -const3 .* (r11 + r22) ); %vh-pol
    scat_cal(:,2,1,ii) = const2 .* c .* (2.0 .* (const1 .* r12 + r21) ...
                  -const3 .* (r11 + r22) ); % hv-pol
end

%-- calculate RCS and phase-difference between the different polarizations

rcs_cal = 10*log10(4*pi* (abs(scat_cal)).^2);  
ratio = scat_cal(:,2,2,:) ./scat_cal(:,1,1,:);
ph_hh_vv = atan2(imag(ratio), real(ratio));

ratio = scat_cal(:,1,2,:) ./scat_cal(:,1,1,:);
ph_vh_vv = atan2(imag(ratio), real(ratio));

ratio = scat_cal(:,2,1,:) ./scat_cal(:,1,1,:);
ph_hv_vv = atan2(imag(ratio), real(ratio));


%--- plot data here --- you can add or modify it to suit !

figure(3)
for ii = 1: nang
    subplot(2,1,1)
    plot(freq, rcs_cal(:,1,1,ii), freq, rcs_cal(:,1,2,ii), ...
        freq, rcs_cal(:,2,1,ii), freq, rcs_cal(:,2,2,ii) )
    set(gca, 'FontSize', 14, 'FontWeight', 'b')
%title('cylinder','FontSize',14, 'FontWeight','b')
    xlabel('Frequency(GHz)','FontSize',14, 'FontWeight','b')
    ylabel('\sigma (dBsm)', 'FontSize',14, 'FontWeight','b')
    ch = get(gca, 'Children');
    set(ch(4),'LineWidth', 2.0, 'LineStyle','-', 'Color','k')
    set(ch(3),'LineWidth', 2.0, 'LineStyle','-', 'Color','b')
    set(ch(2),'LineWidth', 2.0, 'LineStyle','-','Color','g') 
    set(ch(1),'LineWidth', 2.0, 'LineStyle','-','Color','r') 
    legend('VV', 'VH', 'HV', 'HH', 3)
    %axis([fstart fstop -60 10])
    grid
    
    subplot(2,1,2)
    plot(freq, ph_hh_vv*r2d, freq, ph_vh_vv*r2d, freq, ph_hv_vv*r2d)
    set(gca, 'FontSize', 14, 'FontWeight', 'b')
    xlabel('Frequency(GHz)','FontSize',14, 'FontWeight','b')
    ylabel('\phi (deg)', 'FontSize',14, 'FontWeight','b')
    ch = get(gca, 'Children');
    set(ch(3),'LineWidth', 2.0, 'LineStyle','-', 'Color','k')
    set(ch(2),'LineWidth', 2.0, 'LineStyle','-','Color','b') 
    set(ch(1),'LineWidth', 2.0, 'LineStyle','-','Color','r') 
    legend('\phi_{hh-vv}', '\phi_{vh-vv}', '\phi_{hv-vv}', 3)
    grid
    axis([min(freq) max(freq) -180 180])
hold
end
